#ifndef _HIPPO_
#define _HIPPO_

// __POSIX__ is chosen by default if the system is not specified
#if !defined(__XENOMAI__) && !defined(__POSIX__) && !defined(__APPLE__)
    #define __POSIX__
#endif

#if defined(__XENOMAI__)
    #include <stdio.h>
    #include <stdlib.h>
    #include <alarm.h>
    #include <task.h>
    #include <sem.h>
    #include <mutex.h>
#elif defined(__POSIX__)
    #include <stdio.h>
    #include <stdlib.h>
    #include <errno.h>
    #include <unistd.h>
    #include <time.h>
    #include <sys/signal.h>
    #include <sys/time.h>
    #include <pthread.h>
    #include <semaphore.h>
    #include <fcntl.h>
#elif defined(__APPLE__)
    #define NANOSLEEP //For MACOS, timers are managed with nanosleep methods
    #include <stdio.h>
    #include <stdlib.h>
    #include <errno.h>
    #include <unistd.h>
    #include <time.h>
    #include <sys/signal.h>
    #include <sys/time.h>
    #include <pthread.h>
    #include <semaphore.h>
    #include <fcntl.h>
    //#include "mac__APPLE__time.h"
#endif

#if defined(AFFINITY)
    #define _GNU_SOURCE
    #include <sched.h>
    #include <unistd.h>
#endif

#include "data.h"
#include "hippo_app_data.h"
#include "hippo_app_functions.h"

/** \def typedef struct s_fireable Fireable
 *  \brief A structure to represent a element of the ordered list of fireable
 *  transitions
 */
typedef struct s_fireable Fireable;

/** a fireable element
 */
struct s_fireable {
    int state; /** is the transition is fireable */
    int prev; /** previous fireable transition */
    int next; /** next fireable transition */
};

/** \def typedef struct s_enable Enable
 *  \brief A structure to represent a element of the ordered list of enable
 *  transitions
 */
typedef struct s_enable Enable;

/** an enable element
 */
struct s_enable {
    int state; /** is the transition is enable */
    int prev; /** previous enable transition */
    int next; /** next enable transition */
};

/**
 * @def typdeef struct s_setOfActiveAlarms setOfActiveAlarms
 * @brieg A structure to manage the alarms of hippo
 * */
typedef struct s_ActiveAlarm ActiveAlarm;

/**
 * A hippo alarm
 * */
struct s_ActiveAlarm{
    unsigned long long int date;
    int active;
    int prev;
    int next;
};

/**
 * Run the engine
 */
void engine_run();

#endif
