#How to compile an Hippo executable

Hippo is a toolchain for automatically generating code for real-time systems from a high-level specification of its intended behavior. It is based on the formal language [Fiacre](http://projects.laas.fr/fiacre/home.php) and provides methods and tools for generating executable code from it. These tools   produce a faithful executable; meaning that the generated code and the model have exactly the same sequence of observable events, and respect the same timing constraints.

##Prerequisites

You only need an operational frac-hippo compiler (the path to frac is supposed configured) and a C compiler on a Linux machine.

##Compile the model with fiacre-hippo

To compile a model in fiacre (`MyFile.fcr`), go to the directory and call the frac compiler:

`frac MyFile.fcr MyFile.hippo`

Files are generated in `MyFile.hippo`.

##Generate the makefile with config.py

From the repository of your project call the python scipt to configure the makefile:

```
python3 path/config.py <project-name> <options>
```

The options are:

Option | Descripton | Mandatory
--------------- | :--- | ---
--help, -h  | show options | no
--target, -t=\<posix \| xenomai\>  | define the target | yes
--unit, -u = \<x>\<us \| ms \| s> | set the time unit (second by default) | no
--debug, -d = \<inline\|scn> |print debug information during the execution, scn format can be import in play from Tina toolbox | no
--lttng, -l = \<yes> |add lltng | no
--gdb, -g = \<yes> |add -g option flag to generate debug information to be used by GDB debugger| no
--hippolib, -h = \<path> |specify the path (realtive or absolute) of the hippo library| no

For example, to generate the makefile of the doubleclick test for a posix target with a tick of 10 ms and the inline trace mode:

`python3 ../../config.py doubleclick -t=posix -u=10ms -d=inline`


##Compile and execute an hippo model

To compile your model, call the Makefile with `make` or force to cleanup before with 
`make clean`.

To execute your executable you simply to run it.

If you use Xenomai version, you need to run the executable with superuser privileges.


## Additionnal informations for deployment

###Install Xenomai (Mercury)

If you want to used the Xenomai version of Hippo, you can simply install the Mercury skin. To do this, clone from the main repository:

`$ wget https://xenomai.org/downloads/xenomai/stable/xenomai-3.0.9.tar.bz2`
`$ tar xjf xenomai-3.0.9.tar.bz2 `

You need Autotools and Autoconf

`$ sudo apt-get install autotools-dev`
`$ sudo apt-get install autoconf`
`$ sudo apt-get install build-essential`

Then compile the lib

```
$ cd xenomai-3.0.9
$ mkdir $build_root && cd $build_root
$ ../configure --with-core=mercury --enable-smp --enable-pshared --enable-lores-clock --enable-debug=partial
$ sudo make install
```

###Install PREEMPT-RT

To easyly install PREEMPT_RT on Ubuntu 18.04 you can follow the tutorial of [Chenna Kautilya] (https://chenna.me/blog/2020/02/23/how-to-setup-preempt-rt-on-ubuntu-18-04/).

###Use lttng userspace to generate trace

An LTTng-based trace system is available in Hippo. [LTTng](https://lttng.org) is an open source tracing framework for Linux. To use it, you have to  install it first, see [LTTng documentation](https://lttng.org/docs/v2.10/#doc-ubuntu):

```
apt-add-repository ppa:lttng/stable-2.10
apt-get update
apt-get install lttng-tools
apt-get install lttng-modules-dkms
apt-get install liblttng-ust-dev
```

To generate an Hippo executable with LTTng, you have to generate the makefile with the option `--lttng=yes` and compile your project.

Then start the LTTng session :

```
$ lttng-sessiond --daemonize
$ lttng list --userspace
$ lttng create hippo-session
$ lttng enable-event --userspace hippo:clock,hippo:start_one_turn_engine,hippo:end_one_turn_engine
```
and run the application
`$ lttng start`

Close the session with

```
$ lttng stop
$ lttng destroy
```

