# Vefirication of hippo execution

For a given fiacre model :
  * compile it in hippo,
  * run hippo executable in a given delay by writing execution trace in a scn file
  * compile the fiacre model in tts
  * play this tts in PLAY tool of TINA
  * load previously got scn in PLAY to check if the execution trace is playable by the tts model


#### Table of contents

* [Usage](#usage)
* [A short "How to"](#a-short-how-to)
* [Content](#content)
* [Dependencies](#dependencies)
   * [Relative paths, Python modules and Environment variables](#relative-paths-python-modules-and-environment-variables)
   * [Main C](#main-c)
   * [Specific version of frac](#specific-version-of-frac)
   * [Constraints about models in input](#constraints-about-models-in-input)
      * [Hippo requirements](#hippo-requirements)
      * [TTS requirements](#tts-requirements)
      * [Decimal time values](#decimal-time-values)


## Usage
```sh
./fcr_play_scn.py -n <name> -u <unit> -t <timeout>
WARNING : All arguments above, except timeout, are mandatory.

-n <name> : fiacre model name
-u <unit> : time unit, in second
-t <timeout> : timeout of hippo execution (in second), disabled by default. Note : Ctrl+C interruption is handled.
```

Let's get the following example :
```sh
./fcr_play_scn.py -u 0.001 -t 3 -n trains4.fcr
```
Runs test for trains4.fcr with a tick equalt to 0.001s = 1ms.
Timeout is set to 3 seconds, so after running hippo executable for 3 seconds,
the executable is terminated automatically.

## A short "How to"
Let's get the previous example. We want to verify hippo execution of trains4.fcr.
After running the command at line 1, trains4.fcr is compiled to TTS then
to hippo (line 6) by generated a Makefile before (line 3). Generated
hippo executable is named `trains4` and it runs for 3 seconds (`-t 3` option).
Execution trace is saved in a `trains4.scn` file. After terminating hippo
execution `trains4.tts` previously compiled is played in `Play`, a TINA tool
(line 8 to 17). Finally `trains.scn` is loaded (line 18). The scenario
described in `trains4.scn` is played and if an error occurs, an error message is
printed between "loaded..." line (line 18) and "bye" line (line 20).
In the trace bellow, we can see that there is no error, so the scn trace
is playable by the tts model.

```
 1  $ ./fcr_play_scn.py -u 0.001 -t 3 -n trains4.fcr
 2
 3  ** The Makefile is generated
 4  ** Make to compile
 5  rm trains4 | true
 6  make
 7
 8  Play version 3.6.0 -- 07/16/20 -- LAAS/CNRS
 9  parsed net process_train_4_train_3_train_2_train_1_controler_1_barrier_1
10  0.004s
11  
12  initial
13  date: 0
14  state 0: barrier_1_sup controler_1_swaitc train_1_sfar train_2_sfar train_3_sfar train_4_sfar {barrier_1_vstates=0,controler_1_vstates=0,controler_1_vx=0,train_1_vstates=0,train_2_vstates=0,train_3_vstates=0,train_4_vstates=0}
15  enabled: train_1_t0_controler_1_t0 [0,w[ train_2_t0_controler_1_t0 [0,w[ train_3_t0_controler_1_t0 [0,w[ train_4_t0_controler_1_t0 [0,w[
16  firable: train_1_t0_controler_1_t0 train_2_t0_controler_1_t0 train_3_t0_controler_1_t0 train_4_t0_controler_1_t0
17  
18  ? loaded path from trains4.scn
19  
20  ? bye
```

Here we introduce some errors messages that can occur :
* `eval error: transition <transition name> not firable from current state <state name>` :
problems described in "Decimal time values" section or in "unbounded wait intervals"
point of "Hippo requirements" section bellow can throw this error
* `eval error: unknown transition eval` :
indications in "Specific version of frac" section bellow could fix this error
* `eval error: delay not applicable from current state`

Note : be careful of hippo execution issues **that produce any scn trace**.
So scn file in ouput will be empty and **no error message** will be printed by
loading an empty scn file in `Play`.

## Content
* `fcr_play_scn.py` : main program to run the tests
* `main.c` : main for hippo executables
* `readme.md` : useful informations for users
* `*.fcr` : some fiacre models to test. Currently all of
the models in this repository have been passed for the current version of hippo.


## Dependencies
To be able to run the tests, please **be careful** about the indications in
this section.

### Relative paths, Python modules and Environment variables
At the top of the `fcr_play_scn.py` file we have the following few lines :

```py
 1  # Depends on "gen_periodic.py" script of the "measures/exe" directory
 2  sys.path.append("../../../measures")
 3  import exe.gen_periodic as g
 4
 5  # Modify relative paths, overrides gen_periodic's values
 6  g.CONFIG_PATH = "../config.py" # config script to generate hippo Makefile
 7  g.HIPPO_PATH = "../lib/hippo/" # hippo library
 8
 9  FRACPATH = os.getenv("FRACPATH") # FRACPATH environment variable, must be defined
10  assert FRACPATH != None, "FRACPATH environment variable is not defined"
```
* lines 2 and 3 : import gen_periodic.py script from measures/exe directory.
Adapt path at line 2 if necessary.
* lines 6 : path to config.py script. This script is used to generate Makefile
for compilation to hippo executable.
* lines 7 : path to hippo library.
* lines 9 : get the path to the `frac` compiler from the FRACPATH
environment variable. So please define this variable or write directly by hand
the path to `frac` compiler in `fcr_play_scn.py`.
A version of `frac` that is able to compile both fiacre-to-tts and
fiacre-to-hippo is needed.

### Main C
To generate a hippo executable, a `main.c` is mandatory. At least it runs the
hippo engine by calling the `engine_run()` function of hippo.

### Specific version of `frac`
Hippo executable is compiled with `scn` debug option that prints an execution
trace with a "scn format". This **scn ouput is always timed**, even when
the fiacre model in input is not timed. In this case, there can be `PLAY`
errors, so to able to test not timed fiacre **we need a specific version of
`PLAY`** that can get timed scn for not timed models.

### Constraints about models in input
As tested models are compiled both in TTS and hippo, they must satisfy
both fiacre-to-tts and fiacre-to-hippo compilation requirements.

#### Hippo requirements
To be compilable to hippo, models in input must **not have** :
* "Properties" part,
* label (noted "#label") on transition,
* Hippo keywords like "start", "sync", "task" do not be used in fiacre model
as name for process, port, variable, etc.
* C keywords like "register"

Models in input must also **not have** :
* unbounded `wait` intervals : if they are unbounded, hippo use a finite
default value, so errors can occur when trying to play scn trace


#### TTS requirements
To be compilable to hippo TTs, models in input must **not have** hippo task or event.

#### Decimal time values
When there are decimal time values in `wait` intervals, `frac` apply a power
of 10 factor to have interger values for compability with TINA tools.
If it occurs, this factor is applyed in the fiacre-to-tts compilation but
not in the fiacre-to-hippo compilation. So delay in tts and hippo models
will be different and the verification will fail. To avoid this,
we must rewrite by hand decimal delays to integer delays before running
the tests.
